#property script_show_inputs

#include <MT4Orders.mqh> // https://www.mql5.com/ru/code/16006

input datetime inFrom = D'2021.06.01'; // From what date to calculate.

template <typename T>
struct STRUCT
{
protected:
  T Data[];

  int GetPos( const T &Value )
  {
    const int Size = this.GetAmount();
    int Pos = 0;

    while ((Pos < Size) && this[Pos] != Value)
      Pos++;

    if (Pos == Size)
      ::ArrayResize(this.Data, Size + 1);

    return(Pos);
  }

public:
  int GetAmount( void ) const
  {
    return(::ArraySize(this.Data));
  }

  const T operator []( const int Pos ) const
  {
    return(this.Data[Pos]);
  }

  void operator +=( const T Value )
  {
    this.Data[this.GetPos(Value)] = Value;
  }

  int Copy( T &Array[] ) const
  {
    return(ArrayCopy(Array, this.Data));
  }
};

int GetSymbols( string &Symbols[], const datetime From = 0 )
{
  STRUCT<string> Symb;

  for (int i = OrdersHistoryTotal() - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY))
    {
      if (OrderCloseTime() < From)
        break;
      else if ((OrderType() <= OP_SELL) && OrderLots())
        Symb += OrderSymbol();
    }

  return(Symb.Copy(Symbols));
}

void CalcSwaps( const string Symb, const datetime From = 0 )
{
#define OPENTIME_INDEX 0
#define PROFIT_INDEX 1
  double Profits[][2];

  double Profit = 0;
  int Amount = 0;

  for (int i = ArrayResize(Profits, OrdersHistoryTotal()) - 1; i >= 0; i--)
    if (OrderSelect(i, SELECT_BY_POS, MODE_HISTORY))
    {
      if (OrderCloseTime() < inFrom)
        break;
      else if ((OrderType() <= OP_SELL) && OrderLots() && (OrderSymbol() == Symb))
      {
        Profits[Amount][OPENTIME_INDEX] = (double)(OrderOpenTime() % (24 * 3600));

        Profits[Amount][PROFIT_INDEX] = (OrderProfit() + OrderSwap() + OrderCommission()) / OrderLots();

        Profit += Profits[Amount][PROFIT_INDEX];

        Amount++;
      }
    }

  ArrayResize(Profits, Amount);
  ArraySort(Profits);

  Print(Symb + ":\n" + TimeToString(0, TIME_SECONDS) + " Profit =  " + DoubleToString(Profit, 2));

  double MaxProfit = Profit;

  for (int i = Amount - 1; i >= 0; i--)
  {
    Profit -= Profits[i][PROFIT_INDEX];

    if (Profit > MaxProfit)
    {
      MaxProfit = Profit;

      Print(TimeToString((datetime)Profits[i][OPENTIME_INDEX], TIME_SECONDS) +
            " Profit =  " + DoubleToString(Profit, 2) +
            " " + (string)(i - Amount) + "/" + (string)Amount);
    }
  }
}

void OnStart()
{
  string Symbols[];

  for (int i = GetSymbols(Symbols, inFrom) - 1; i >= 0; i--)
  {
    Print(i);

    CalcSwaps(Symbols[i], inFrom);
  }
}